/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(Ximera, injector, Promise, config) {

    const bunyan = require('bunyan');

    const _init = Promise.method(function() {

        var streams = [];

        if (config.log) {
            if (config.log.console) {
                streams.push({
                    stream: process.stdout,
                    level: config.log.console.level || 'warn'
                });
            }

            if (config.log.file) {
                streams.push({
                    type: 'file',
                    path: config.log.file.path,
                    level: config.log.file.level,
                    name: config.log.file.name
                });
            }

            if (config.log.rotating) {
                streams.push({
                    type: 'rotating-file',
                    path: config.log.rotating.path,
                    level: config.log.rotating.level,
                    name: config.log.rotating.name,
                    period: config.log.rotating.period || '1d',
                    count: config.log.rotating.count || 7
                });
            }

            if (config.log.syslog && process.env.NODE_ENV !== 'test') {
                let bsyslog = require('bunyan-syslog-udp');

                streams.push({
                    type: 'raw',
                    level: config.log.syslog.level,
                    stream: bsyslog.createBunyanStream(config.log.syslog)
                });
            }

            if (config.log.elk && process.env.NODE_ENV !== 'test') {

                streams.push({
                    type: "raw",
                    stream: require('bunyan-logstash-tcp').createStream({
                        host: config.log.elk.host,
                        port: config.log.elk.port,
                        level: config.log.elk.level
                    })
                });

            }

            Ximera.log = bunyan.createLogger({
                name: config.log.name || 'system',
                serializers: bunyan.stdSerializers,
                streams: streams
            });

            // Override window.console to automatically route console.log to router
            console.log = function(...args) {
                return Ximera.log.debug(...args);
            };

        } else {
            Ximera.log = {
                debug: function() {},
                warn: function() {},
                info: function() {},
                trace: function() {},
                error: function() {}
            };
        }

        // Register the system logger
        console.log("Registering the logger");
        return injector.register('log', Ximera.log, {
            concrete: true
        });

    });

    return Promise.resolve({
        init: _init
    });

};